#ifndef XATOM_H
#define XATOM_H 1

/* THIS IS A GENERATED FILE
 *
 * Do not change!  Changing this file implies a protocol change!
 */

#define XA_PRIMARY ((Atom)1)
#define XA_SECONDARY ((Atom)2)
#define XA_ARC ((Atom)3)
#define XA_ATOM ((Atom)4)
#define XA_BITMAP ((Atom)5)
#define XA_CARDINAL ((Atom)6)
#define XA_COLORMAP ((Atom)7)
#define XA_CURSOR ((Atom)8)
#define XA_CUT_BUFFER0 ((Atom)9)
#define XA_CUT_BUFFER1 ((Atom)10)
#define XA_CUT_BUFFER2 ((Atom)11)
#define XA_CUT_BUFFER3 ((Atom)12)
#define XA_CUT_BUFFER4 ((Atom)13)
#define XA_CUT_BUFFER5 ((Atom)14)
#define XA_CUT_BUFFER6 ((Atom)15)
#define XA_CUT_BUFFER7 ((Atom)16)
#define XA_DRAWABLE ((Atom)17)
#define XA_FONT ((Atom)18)
#define XA_INTEGER ((Atom)19)
#define XA_PIXMAP ((Atom)20)
#define XA_POINT ((Atom)21)
#define XA_RECTANGLE ((Atom)22)
#define XA_RESOURCE_MANAGER ((Atom)23)
#define XA_RGB_COLOR_MAP ((Atom)24)
#define XA_RGB_BEST_MAP ((Atom)25)
#define XA_RGB_BLUE_MAP ((Atom)26)
#define XA_RGB_DEFAULT_MAP ((Atom)27)
#define XA_RGB_GRAY_MAP ((Atom)28)
#define XA_RGB_GREEN_MAP ((Atom)29)
#define XA_RGB_RED_MAP ((Atom)30)
#define XA_STRING ((Atom)31)
#define XA_VISUALID ((Atom)32)
#define XA_WINDOW ((Atom)33)
#define XA_WM_COMMAND ((Atom)34)
#define XA_WM_HINTS ((Atom)35)
#define XA_WM_CLIENT_MACHINE ((Atom)36)
#define XA_WM_ICON_NAME ((Atom)37)
#define XA_WM_ICON_SIZE ((Atom)38)
#define XA_WM_NAME ((Atom)39)
#define XA_WM_NORMAL_HINTS ((Atom)40)
#define XA_WM_SIZE_HINTS ((Atom)41)
#define XA_WM_ZOOM_HINTS ((Atom)42)
#define XA_MIN_SPACE ((Atom)43)
#define XA_NORM_SPACE ((Atom)44)
#define XA_MAX_SPACE ((Atom)45)
#define XA_END_SPACE ((Atom)46)
#define XA_SUPERSCRIPT_X ((Atom)47)
#define XA_SUPERSCRIPT_Y ((Atom)48)
#define XA_SUBSCRIPT_X ((Atom)49)
#define XA_SUBSCRIPT_Y ((Atom)50)
#define XA_UNDERLINE_POSITION ((Atom)51)
#define XA_UNDERLINE_THICKNESS ((Atom)52)
#define XA_STRIKEOUT_ASCENT ((Atom)53)
#define XA_STRIKEOUT_DESCENT ((Atom)54)
#define XA_ITALIC_ANGLE ((Atom)55)
#define XA_X_HEIGHT ((Atom)56)
#define XA_QUAD_WIDTH ((Atom)57)
#define XA_WEIGHT ((Atom)58)
#define XA_POINT_SIZE ((Atom)59)
#define XA_RESOLUTION ((Atom)60)
#define XA_COPYRIGHT ((Atom)61)
#define XA_NOTICE ((Atom)62)
#define XA_FONT_NAME ((Atom)63)
#define XA_FAMILY_NAME ((Atom)64)
#define XA_FULL_NAME ((Atom)65)
#define XA_CAP_HEIGHT ((Atom)66)
#define XA_WM_CLASS ((Atom)67)
#define XA_WM_TRANSIENT_FOR ((Atom)68)

#define XA_LAST_PREDEFINED ((Atom)68)
#endif /* XATOM_H */
