/* zutil.h -- internal interface and configuration of the compression library
 * Copyright (C) 1995-2013 Jean-loup Gailly.
 * For conditions of distribution and use, see copyright notice in zlib.h
 */

/* WARNING: this file should *not* be used by applications. It is
   part of the implementation of the compression library and is
   subject to change. Applications should only use zlib.h.
 */

/* @(#) $Id$ */

#ifndef ZUTIL_H
#define ZUTIL_H

#ifdef HAVE_HIDDEN
#define ZLIB_INTERNAL __attribute__((visibility("hidden")))
#else
#define ZLIB_INTERNAL
#endif

#include "zlib.h"

#if defined(STDC) && !defined(Z_SOLO)
#if !(defined(_WIN32_WCE) && defined(_MSC_VER))
#include <stddef.h>
#endif
#include <string.h>
#include <stdlib.h>
#endif

#ifdef Z_SOLO
typedef long ptrdiff_t; /* guess -- will be caught if guess is wrong */
#endif

#ifndef local
#define local static
#endif
/* compile with -Dlocal if your debugger can't find static symbols */

typedef unsigned char uch;
typedef uch FAR uchf;
typedef unsigned short ush;
typedef ush FAR ushf;
typedef unsigned long ulg;

extern z_const char *const z_errmsg[10]; /* indexed by 2-zlib_error */
/* (size given to avoid silly warnings with Visual C++) */

#define ERR_MSG(err) z_errmsg[Z_NEED_DICT - (err)]

#define ERR_RETURN(strm, err) \
	return (strm->msg = ERR_MSG(err), (err))
/* To be used only when the state is known to be valid */

/* common constants */

#ifndef DEF_WBITS
#define DEF_WBITS MAX_WBITS
#endif
/* default windowBits for decompression. MAX_WBITS is for compression only */

#if MAX_MEM_LEVEL >= 8
#define DEF_MEM_LEVEL 8
#else
#define DEF_MEM_LEVEL MAX_MEM_LEVEL
#endif
/* default memLevel */

#define STORED_BLOCK 0
#define STATIC_TREES 1
#define DYN_TREES 2
/* The three kinds of block type */

#define MIN_MATCH 3
#define MAX_MATCH 258
/* The minimum and maximum match lengths */

#define PRESET_DICT 0x20 /* preset dictionary flag in zlib header */

/* target dependencies */

#if defined(MSDOS) || (defined(WINDOWS) && !defined(WIN32))
#define OS_CODE 0x00
#ifndef Z_SOLO
#if defined(__TURBOC__) || defined(__BORLANDC__)
#if (__STDC__ == 1) && (defined(__LARGE__) || defined(__COMPACT__))
/* Allow compilation with ANSI keywords only enabled */
void _Cdecl farfree(void *block);
void *_Cdecl farmalloc(unsigned long nbytes);
#else
#include <alloc.h>
#endif
#else /* MSC or DJGPP */
#include <malloc.h>
#endif
#endif
#endif

#ifdef AMIGA
#define OS_CODE 0x01
#endif

#if defined(VAXC) || defined(VMS)
#define OS_CODE 0x02
#define F_OPEN(name, mode) \
	fopen((name), (mode), "mbc=60", "ctx=stm", "rfm=fix", "mrs=512")
#endif

#if defined(ATARI) || defined(atarist)
#define OS_CODE 0x05
#endif

#ifdef OS2
#define OS_CODE 0x06
#if defined(M_I86) && !defined(Z_SOLO)
#include <malloc.h>
#endif
#endif

#if defined(MACOS) || defined(TARGET_OS_MAC)
#define OS_CODE 0x07
#ifndef Z_SOLO
#if defined(__MWERKS__) && __dest_os != __be_os && __dest_os != __win32_os
#include <unix.h> /* for fdopen */
#else
#ifndef fdopen
#define fdopen(fd, mode) NULL /* No fdopen() */
#endif
#endif
#endif
#endif

#ifdef TOPS20
#define OS_CODE 0x0a
#endif

#ifdef WIN32
#ifndef __CYGWIN__ /* Cygwin is Unix, not Win32 */
#define OS_CODE 0x0b
#endif
#endif

#ifdef __50SERIES /* Prime/PRIMOS */
#define OS_CODE 0x0f
#endif

#if defined(_BEOS_) || defined(RISCOS)
#define fdopen(fd, mode) NULL /* No fdopen() */
#endif

#if (defined(_MSC_VER) && (_MSC_VER > 600)) && !defined __INTERIX
#if defined(_WIN32_WCE)
#define fdopen(fd, mode) NULL /* No fdopen() */
#ifndef _PTRDIFF_T_DEFINED
typedef int ptrdiff_t;
#define _PTRDIFF_T_DEFINED
#endif
#else
#define fdopen(fd, type) _fdopen(fd, type)
#endif
#endif

#if defined(__BORLANDC__) && !defined(MSDOS)
#pragma warn - 8004
#pragma warn - 8008
#pragma warn - 8066
#endif

/* provide prototypes for these when building zlib without LFS */
#if !defined(_WIN32) && \
	(!defined(_LARGEFILE64_SOURCE) || _LFS64_LARGEFILE - 0 == 0)
ZEXTERN uLong ZEXPORT adler32_combine64 OF((uLong, uLong, z_off_t));
ZEXTERN uLong ZEXPORT crc32_combine64 OF((uLong, uLong, z_off_t));
#endif

/* common defaults */

#ifndef OS_CODE
#define OS_CODE 0x03 /* assume Unix */
#endif

#ifndef F_OPEN
#define F_OPEN(name, mode) fopen((name), (mode))
#endif

/* functions */

#if defined(pyr) || defined(Z_SOLO)
#define NO_MEMCPY
#endif
#if defined(SMALL_MEDIUM) && !defined(_MSC_VER) && !defined(__SC__)
/* Use our own functions for small and medium model with MSC <= 5.0.
  * You may have to use the same strategy for Borland C (untested).
  * The __SC__ check is for Symantec.
  */
#define NO_MEMCPY
#endif
#if defined(STDC) && !defined(HAVE_MEMCPY) && !defined(NO_MEMCPY)
#define HAVE_MEMCPY
#endif
#ifdef HAVE_MEMCPY
#ifdef SMALL_MEDIUM /* MSDOS small or medium model */
#define zmemcpy _fmemcpy
#define zmemcmp _fmemcmp
#define zmemzero(dest, len) _fmemset(dest, 0, len)
#else
#define zmemcpy memcpy
#define zmemcmp memcmp
#define zmemzero(dest, len) memset(dest, 0, len)
#endif
#else
void ZLIB_INTERNAL zmemcpy OF((Bytef * dest, const Bytef* source, uInt len));
int ZLIB_INTERNAL zmemcmp OF((const Bytef* s1, const Bytef* s2, uInt len));
void ZLIB_INTERNAL zmemzero OF((Bytef * dest, uInt len));
#endif

/* Diagnostic functions */
#ifdef DEBUG
#include <stdio.h>
extern int ZLIB_INTERNAL z_verbose;
extern void ZLIB_INTERNAL z_error OF((char *m));
#define Assert(cond, msg)          \
	{                              \
		if (!(cond)) z_error(msg); \
	}
#define Trace(x)                       \
	{                                  \
		if (z_verbose >= 0) fprintf x; \
	}
#define Tracev(x)                     \
	{                                 \
		if (z_verbose > 0) fprintf x; \
	}
#define Tracevv(x)                    \
	{                                 \
		if (z_verbose > 1) fprintf x; \
	}
#define Tracec(c, x)                         \
	{                                        \
		if (z_verbose > 0 && (c)) fprintf x; \
	}
#define Tracecv(c, x)                        \
	{                                        \
		if (z_verbose > 1 && (c)) fprintf x; \
	}
#else
#define Assert(cond, msg)
#define Trace(x)
#define Tracev(x)
#define Tracevv(x)
#define Tracec(c, x)
#define Tracecv(c, x)
#endif

#ifndef Z_SOLO
voidpf ZLIB_INTERNAL zcalloc OF((voidpf opaque, unsigned items,
								 unsigned size));
void ZLIB_INTERNAL zcfree OF((voidpf opaque, voidpf ptr));
#endif

#define ZALLOC(strm, items, size) \
	(*((strm)->zalloc))((strm)->opaque, (items), (size))
#define ZFREE(strm, addr) (*((strm)->zfree))((strm)->opaque, (voidpf)(addr))
#define TRY_FREE(s, p)      \
	{                       \
		if (p) ZFREE(s, p); \
	}

/* Reverse the bytes in a 32-bit value */
#define ZSWAP32(q) ((((q) >> 24) & 0xff) + (((q) >> 8) & 0xff00) + \
					(((q)&0xff00) << 8) + (((q)&0xff) << 24))

#endif /* ZUTIL_H */
