#ifndef B3_SOUND_C_API_H
#define B3_SOUND_C_API_H

//todo: create a sound C-API

//create sound engine
//destroy sound engine
//getSoundSource
//startSound
//releaseSound
//etc

enum B3_SOUND_SOURCE_TYPE
{
	B3_SOUND_SOURCE_SINE_OSCILLATOR = 1,
	B3_SOUND_SOURCE_SAW_OSCILLATOR,
	B3_SOUND_SOURCE_WAV_FILE,
};

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __cplusplus
}
#endif

#endif  ///B3_SOUND_C_API_H
