#ifndef _LOAD_SHADER_H
#define _LOAD_SHADER_H

#include "OpenGLInclude.h"

#ifdef __cplusplus
extern "C"
{
#endif  //__cplusplus

	GLuint gltLoadShaderPair(const char *szVertexProg, const char *szFragmentProg);

#ifdef __cplusplus
}
#endif  //__cplusplus

#endif  //_LOAD_SHADER_H
