//this file is autogenerated using stringify.bat (premake --stringify) in the build folder of this project
static const char* pointsFragmentShader= \
"#version 150\n"
"in vec4 colourV;\n"
"out vec4 fragColour;\n"
"void main(void)\n"
"{\n"
"    fragColour = colourV;\n"
"}\n"
;
